import 'package:flutter/material.dart';

extension MediaQueryExt on BuildContext {

  Size get mediaQuerySize => MediaQuery.of(this).size;

  double get width => MediaQuery.of(this).size.width;

  double get height => MediaQuery.of(this).size.height;

  EdgeInsets get mediaQueryPadding => MediaQuery.of(this).padding;

  EdgeInsets get mediaQueryViewPadding => MediaQuery.of(this).viewPadding;

  EdgeInsets get mediaQueryViewInsets => MediaQuery.of(this).viewInsets;

  Orientation get orientation => MediaQuery.of(this).orientation;

  bool get isLandscape => orientation == Orientation.landscape;

  bool get isPortrait => orientation == Orientation.portrait;

  bool get alwaysUse24HourFormat => MediaQuery.of(this).alwaysUse24HourFormat;

  double get devicePixelRatio => MediaQuery.of(this).devicePixelRatio;

  Brightness get platformBrightness => MediaQuery.of(this).platformBrightness;

  double get textScaleFactor => MediaQuery.of(this).textScaleFactor;

  double get mediaQueryShortestSide => mediaQuerySize.shortestSide;

  /// True if the current device is Phone
  bool get isPhone => (mediaQueryShortestSide < 600);

  /// 600dp: a 7” tablet (600x1024 mdpi).
  bool get isSmallTablet => (mediaQueryShortestSide >= 600);

  /// 720dp: a 10” tablet (720x1280 mdpi, 800x1280 mdpi, etc).
  bool get isLargeTablet => (mediaQueryShortestSide >= 720);

  /// True if the current device is Tablet
  bool get isTablet => isSmallTablet || isLargeTablet;
}

extension NavigatorExt on BuildContext {
  Future<T> push<T>(Route<T> route) => Navigator.push(this, route);

  void pop<T extends Object>([T result]) => Navigator.pop(this, result);

  Future<T> pushNamed<T>(String routeName, {Object arguments}) =>
      Navigator.pushNamed<T>(this, routeName, arguments: arguments);

  Future<T> pushReplacement<T>(Route<T> route) => Navigator.pushReplacement(this, route);

  bool canPop() => Navigator.canPop(this);

  void popUntil(RoutePredicate predicate) =>
      Navigator.popUntil(this, predicate);
}

extension ThemeExt on BuildContext {

  ThemeData get theme => Theme.of(this);

  TextTheme get textTheme => Theme.of(this).textTheme;

  TextTheme get primaryTextTheme => Theme.of(this).primaryTextTheme;

  TextTheme get accentTextTheme => Theme.of(this).accentTextTheme;

  BottomAppBarTheme get bottomAppBarTheme => Theme.of(this).bottomAppBarTheme;

  BottomSheetThemeData get bottomSheetTheme => Theme.of(this).bottomSheetTheme;

  Color get backgroundColor => Theme.of(this).backgroundColor;

  Color get primaryColor => Theme.of(this).primaryColor;

  Color get buttonColor => Theme.of(this).buttonColor;

  Color get scaffoldBackgroundColor => Theme.of(this).scaffoldBackgroundColor;

  AppBarTheme get appBarTheme => Theme.of(this).appBarTheme;

  TargetPlatform get platform => Theme.of(this).platform;

  bool get isAndroid => this.platform == TargetPlatform.android;

  bool get isIOS => this.platform == TargetPlatform.iOS;

  bool get isMacOS => this.platform == TargetPlatform.macOS;

  bool get isWindows => this.platform == TargetPlatform.windows;

  bool get isFuchsia => this.platform == TargetPlatform.fuchsia;

  bool get isLinux => this.platform == TargetPlatform.linux;

  TextStyle get headline1 => textTheme.headline1;

  TextStyle get headline2 => textTheme.headline2;

  TextStyle get headline3 => textTheme.headline3;

  TextStyle get headline4 => textTheme.headline4;

  TextStyle get headline5 => textTheme.headline5;

  TextStyle get headline6 => textTheme.headline6;

  TextStyle get subtitle1 => textTheme.subtitle1;

  TextStyle get bodyText1 => textTheme.bodyText1;

  TextStyle get bodyText2 => textTheme.bodyText2;

  TextStyle get caption => textTheme.caption;

  TextStyle get button => textTheme.button;

  TextStyle get subtitle2 => textTheme.subtitle2;

  TextStyle get overLine => textTheme.overline;
}

extension ScaffoldExt on BuildContext {
  ScaffoldFeatureController<SnackBar, SnackBarClosedReason> showSnackBar(
      SnackBar snackBar) =>
      Scaffold.of(this).showSnackBar(snackBar);

  void removeCurrentSnackBar(
      {SnackBarClosedReason reason = SnackBarClosedReason.remove}) =>
      Scaffold.of(this).removeCurrentSnackBar(reason: reason);

  void hideCurrentSnackBar(
      {SnackBarClosedReason reason = SnackBarClosedReason.hide}) =>
      Scaffold.of(this).hideCurrentSnackBar(reason: reason);

  void openDrawer() => Scaffold.of(this).openDrawer();

  void openEndDrawer() => Scaffold.of(this).openEndDrawer();

  void showBottomSheet(WidgetBuilder builder,
      {Color backgroundColor,
        double elevation,
        ShapeBorder shape,
        Clip clipBehaviour}) =>
      Scaffold.of(this).showBottomSheet(builder,
          backgroundColor: backgroundColor,
          elevation: elevation,
          shape: shape,
          clipBehavior: clipBehaviour);
}

extension ModalRouteExt<T> on BuildContext {

  ModalRoute<T> get modalRoute => ModalRoute.of<T>(this);

  RouteSettings get routeSettings => modalRoute?.settings;

}